classdef DaEditor < handle
% matlab editor wrapper
%% DESCRIPTION
%     DAEDITOR Summary of this class goes here
%       Detailed explanation goes here
%
%     pos =^= matlabs index, von text = 1xn
%     idx =^= matlabs position, von [line, column]
%% VERSIONING
%             Author: Andreas Justin
%             Matlab: 8.2.0.701 (R2013b)
%  Required Products: -
%
%% REVISIONS
% V1.0 | 2013-11-06 | Andreas Justin    | Ersterstellung
% V1.1 | 2014-10-23 | Andreas Justin    | regexRep startSearch und stopSearch hinzugefgt
% 
%
% See also
%     edhandle = com.mathworks.mlservices.MLEditorServices
%     cur_file = edhandle.getEditorApplication.getActiveEditor.getLongName;
%% --------------------------------------------------------------------------------------------
%% Properties
properties
    mEditor            % matlab Editor: matlab.desktop.editor.Document
    mJavaEditor        % matlab Editor: com.mathworks.mde.editor.MatlabEditor
    callbacks          % Objekt welches die Callbacks vom editor handlet: daEditor.DaEditorCallbacks
    findAndReplace     % Objekt fr besseres find and replace: daEditor.FindAndReplace
end

properties (Hidden = true)
    jComponents
    jComponentsName
    jComponentsAccess
    breakpointbar
end

properties (SetAccess = private)
    COLOR_OFF = grafik.Color.valueOfMatlab([0.94118,0.94118,0.94118,1]); % Color der Sidebar (hellgrau)
    COLOR_ON =  grafik.Color.valueOfRGB255([179,203,111,255]); % Color der Sidebar grn

    ctrlMousePos
end
properties (Dependent = true)
    selection          % 1x4 Vector [start Row, start Col, end Row, end Col] obj.mEditor.Selection
    selectionPos       % 1x2 Vector [start pos, end pos]
    cellTxt            % 1xn Cell Matrix n <- anzahl der Zeilen
    filename           % full name of file (D:\pfad\file.m)
    hEditorPane        % aktuelles java fenster: 
                       % javahandle_withcallbacks.com.mathworks.mde.editor.EditorSyntaxTextPane
end 
%% Methods
methods % constructor, getter, setter
    function obj = DaEditor(mFile)
        if nargin < 1 || isempty(mFile);
            edhandle = com.mathworks.mlservices.MLEditorServices;
            lName = edhandle.getEditorApplication.getActiveEditor.getLongName;
            mEditor_ = matlab.desktop.editor.openDocument(char(lName));
        elseif ischar(mFile)
            mEditor_ = matlab.desktop.editor.openDocument(mFile);
        elseif isa(mFile,'com.mathworks.mde.editor.MatlabEditor')
            mEditor_ = matlab.desktop.editor.Document.findEditor(mFile.getLongName);
        end

        obj.mEditor = mEditor_;
        obj.mJavaEditor = obj.mEditor.JavaEditor;
        obj.callbacks = daEditor.DaEditorCallbacks(obj);
        obj.findAndReplace = daEditor.FindAndReplace(obj);

        [obj.jComponentsName, obj.jComponentsAccess, obj.jComponents] = obj.findAllComponents(...
            obj.hEditorPane.getParent.getParent);
    end
    function cellTxt = get.cellTxt(obj)
        % 1xn Cell eine cell pro zeile im editor
        cellTxt = regexp(obj.getText,'\n','split')';
    end
    function selectionPos = get.selectionPos(obj)
        % [startPos,endPos] von der Selection
        posS = obj.idx2pos(obj.mEditor.Selection(1),obj.mEditor.Selection(2));
        posE = obj.idx2pos(obj.mEditor.Selection(3),obj.mEditor.Selection(4));
        selectionPos = [posS,posE];
    end
    function selection = get.selection(obj)
        % [start Row, start Col, end Row, end Col] von selection
        selection = obj.mEditor.Selection;
    end
    function filename = get.filename(obj)
        % fullpath
        filename = obj.mEditor.Filename;
    end
    function hEditorPane = get.hEditorPane(obj)
        % aktuelles java Frame vom editor
        hEditorPane = Inf;
        a = matlab.desktop.editor.EditorUtils.getJavaEditorApplication;
        for jj = 0:a.getOpenEditors.size-1
            lName = a.getOpenEditors.get(jj).getLongName; 
            if strcmp(lName,obj.filename)
                b = a.getOpenEditors.get(jj).getComponent.getComponent(0).getComponent(0).getComponent(0).getComponent(0).getComponent(1).getComponent(0).getComponent(0);
                hEditorPane = handle(b,'CallbackProperties');
                return
            end
        end
    end
end
methods % textManipulation
    function regexRep(obj,expr,rep,ignoreCase, startSearch, stopSearch)
        % startSearch / stopSearch ... limitiert den suchrange 
        %                              performance boost, aber auch fr MESRC soll der richtige Range ersetzt werden 
        %                              (es kann ja sein, dass in einem Kommentar ein MESR-Command steht als Doku).
        %                              Muss pos sein (kein idx)
        if nargin < 4 || isempty(ignoreCase);   ignoreCase = false; end
        if nargin < 5                          startSearch = [];    end
        if nargin < 6                           stopSearch = [];    end
        if (numel(startSearch) > 1) || (numel(stopSearch) > 1); error('wos mochstn'); end
        
        [posS, posE] = obj.getTextPosByExpr(expr,ignoreCase, startSearch,stopSearch);
        if numel(posS) ~= 1
            [posS, posE] = obj.getTextPosByExpr(regexprep(expr,'\\\\','\'),ignoreCase, startSearch,stopSearch);
            % durch schlechtes design, kann es vorkommen, dass e.g. "\\(" ankommt es aber "\(" lauten sollte.
            % dabei ist aber wichtig dass e.g. "D:\asdf" escaped wird -> "D:\\asdf"
            if numel(posS) ~= 1
                error('DaEditor:wrongState','expr "%s" wurde %d mal gefunden!',expr, numel(posS))
            end
        end
        %% TODO erweiterung auf alle und bei %asdf% Ersetzung nur die aktuelle Zeile bercksichtigen
        obj.repText(rep,posS-1,posE);
    end
    function txt = regex(obj,expr,ignoreCase)
        if nargin < 3 || isempty(ignoreCase);   ignoreCase = false; end
         if ignoreCase
            txt = regexpi(obj.getText,expr,'match','once');
         else
            txt = regexp(obj.getText,expr,'match','once');
        end
    end

    function repText(obj,rep,posS,posE)
         obj.mJavaEditor.replaceText(rep,posS,posE);
    end
end
methods % get Methods
    function shortName = getShortName(obj)
        shortName = regexp(obj.filename,'(?<=[\/\\])\w+\.m$','match','once');
    end
    function longName = getLongName(obj)
        longName = obj.filename;
    end
    function txt = getTextByExpr(obj, expr)
        [posS,posE] = obj.getTextPosByExpr(expr);
        txt = obj.getText(posS,posE);
    end
    function txt = getText(obj,posS,posE)
        txt = obj.mEditor.Text;
        if nargin > 1 && ~isempty(posS)
            if numel(posS) > 1; error('getText:multipleText','es wurde mehr als ein Textblock auf einmal gefordert; %d auf einmal',numel(posS)); end
            if nargin < 3 || isempty(posE); posE = posS;    end
            txt = txt(posS:posE);
        end
    end
    function txt = getTextByLine(obj,lin)
      if isscalar(lin)
          txt = obj.cellTxt{lin};
      else
          txt = obj.cellTxt(lin);
      end
    end
    function [posS, posE] = getSectionPosByPos(obj,pos)
        if pos < 1; pos = 1; end
        linesOfSections = obj.getAllLinesOfSections;
        posOfSections = obj.idx2pos(linesOfSections);
        posS = posOfSections(find(posOfSections <= pos, 1, 'last'));
        posE = posOfSections(find(posOfSections > pos, 1,  'first')) - 2;
        if isempty(posE)
            posE = numel(obj.getText);
        end
    end
    function [posS, posE] = getSectionPosByLine(obj,lin)
        if lin < 1; lin = 1; end
        pos = obj.idx2pos(lin);
        [posS, posE] = obj.getSectionPosByPos(pos);
    end
    function [lin,col] = pos2idx(obj,pos)
        for ii = 1:numel(pos)
            [lin(ii,1),col(ii,1)] = matlab.desktop.editor.indexToPositionInLine(obj.mEditor,pos(ii));
        end
    end
    function pos = idx2pos(obj,lin,col)
        if nargin < 3 || isempty(col); col = ones(size(lin)); end
        if lin < 1; lin = 1; end

        pos = zeros(size(lin));
        for ii = 1:numel(lin)
            pos(ii) = matlab.desktop.editor.positionInLineToIndex(obj.mEditor,lin(ii),col(ii));
        end
    end
    function [pos,posE] = getTextPosByExpr(obj,expr,ignoreCase, startSearch,stopSearch)
        % startSearch / stopSearch ... limitiert den suchrange
        %                              performance boost, aber auch fr MESRC soll der richtige Range ersetzt werden 
        %                              (es kann ja sein, dass in einem Kommentar ein MESR-Command steht als Doku).
        %                              Muss pos sein (kein idx)
        txt = obj.getText;
        
        if nargin < 3 || isempty(ignoreCase);  ignoreCase  = true;        end
        if nargin < 4 || isempty(startSearch); startSearch = 1;           end
        if nargin < 5 || isempty(stopSearch);  stopSearch  = numel(txt)-1;  end

        txt = txt(startSearch:stopSearch);
        
        if ignoreCase
            pos = regexpi(txt,expr);
            posTxt = regexpi(txt,expr,'match','once');
        else
            pos = regexp(txt,expr);
            posTxt = regexp(txt,expr,'match','once');
        end
        pos = (startSearch - 1) + pos;
        posE = pos + numel(posTxt) - 1;
    end
    function sectionText = getTextOfSectionByPos(obj,pos)
        [posS, posE] = getSectionPosByPos(obj,pos);
        sectionText = obj.getText(posS:posE);
    end
end
methods % set Methods
    function obj = setText(obj,txt)
        obj.mEditor.Text = txt;
    end
    function obj = setTextAtPos(obj,txt,posS,posE)
        [a,b] = obj.pos2idx(posS);
        [c,d] = obj.pos2idx(posE);
        obj.mEditor.Selection = [a,b,c,d];
        txtE = obj.getText;
        txtE = [txtE(1:posS-1),txt,txtE(posE+1:end)];
        obj.setText(txtE);
    end
    function obj = insertTextAtPos(obj,txt,pos)
        % inserts text at pos(1) (line) pos(2) col
        % todo: pos(3:4) markierung ersetzen
        obj.mEditor.insertTextAtPositionInLine(txt,pos(1),pos(2));
    end

    function setKeyPressedCallback(obj)
        error('this hinders matlabs tab completion')
%             set(obj.hEditorPane,'KeyPressedCallback',@(src,evnt) daEditor.DaEditor.keyPressedCallback(src,evnt,obj))
    end
    function setKeyReleasedCallback(obj,reset)
        if nargin < 2 || isempty(reset);    reset = false;  end
        if ~reset
            set(obj.hEditorPane,'KeyReleasedCallback',@(src,evnt) daEditor.DaEditorCallbacks.keyReleasedCallback(src,evnt,obj))
        else
            set(obj.hEditorPane,'KeyReleasedCallback','')
        end

        obj.colorizeBreakPointBar(obj.COLOR_ON);
    end      
    function setMouseReleasedCallback(obj,reset)
        if nargin < 2 || isempty(reset);    reset = false;  end
        if ~reset
            set(obj.hEditorPane,'MouseReleasedCallback',@(src,evnt) daEditor.DaEditorCallbacks.mouseReleasedCallback(src,evnt,obj))
        else
            set(obj.hEditorPane,'MouseReleasedCallback','')
        end

        obj.colorizeBreakPointBar(obj.COLOR_ON);
    end
end
methods % Navigation
    function obj = goToPosInLine(obj,lin,col)
        obj.mEditor.goToPositionInLine(lin,col);
    end
    function obj = goToLine(obj,lin)
        obj.mEditor.goToPositionInLine(lin,1);
    end
    function obj = goToPos(obj,pos)
        [lin,col] = obj.pos2idx(pos);
        obj.goToPosInLine(lin,col);
    end
end
methods % interactions
    function obj = selectLine(obj,lin)
        obj.goToLine(lin);
    end
    function obj = selectLinCol(obj,sL,sC,eL,eC)
        obj.mEditor.Selection = [sL,sC,eL,eC];
    end
    function obj = alignSelectionBySingleChar(obj,txt)
        if numel(txt) > 1 || ~ischar(txt); error('txt has to be a 1x1 char'); end
        pos = regexp(obj.mEditor.SelectedText,['\',txt]);
        if isempty(pos); return; end
        [lin,col] = obj.pos2idx(pos);
        idx = getIdxOfDoubleEntries(obj,lin);
        for ii = 1:numel(lin)

        end
    end
end
methods (Access = private)
    function linesOfSections = getAllLinesOfSections(obj)
        linesOfSections = find(util.SearchCell(obj.cellTxt,'^\s*%%\s+',[],[],true));
        if ~any(linesOfSections == 1);  linesOfSections = [1;linesOfSections];  end
    end
    function [names, access, component] = findAllComponents(obj,jMainPane,names,access,component,access_base)
        cmps = jMainPane.getComponents;
        if nargin == 2;
            names = {};
            access = {};
            access_base = [];
            component = {};
        end
        for ii = 1:length(cmps)
            %         c = regexp(char(cmps(ii).toString),'\.([^\[\.]+)\[','tokens');
            c = regexp(char(cmps(ii).class),'\.([^\[\.]+$)','tokens');
            names{end+1} = c{1}{1};
            access{end+1} = [access_base ii-1];
            component{end+1} = cmps(ii);
            [names, access, component] = obj.findAllComponents(jMainPane.getComponent(ii-1),names,access,component,access{end});
        end
        names = names(:);
        access = access(:);
        component = component(:);
    end
    function idx = getIdxOfDoubleEntries(obj,val)
        u = unique(val);
        n = histc(val,u);
        idx = [];
        for ii = find(n>1)'
            idx = [idx,find(val==u(ii))];
        end
    end
    function colorizeBreakPointBar(obj,color)
        bpView = [];
        try
            bpView = obj.jComponents{strcmpi(obj.jComponentsName,'BreakpointView$2')};
            bpView.set('background',color.toJava);
        catch
            if isempty(bpView)
                fprintf('\nno breakpoint bar found, file not safed yet.\n')
            end
        end
    end
end
methods (Hidden = true)
    function bool = isInRoot(obj)
        bool = false;
        shName = obj.mJavaEditor.getShortName();
        shName = regexprep(char(shName),'\.m$','','once');

        daEditor_map = getappdata(0,'daEditor_map');
         if isa(daEditor_map,'containers.Map') && daEditor_map.isKey(shName)
            bool = true;
            dE = daEditor_map(shName);
            if isa(dE.hEditorPane,'double') && isinf(dE.hEditorPane)
                bool = false;
                daEditor_map.remove(shName);
                setappdata(0,'daEditor_map',daEditor_map);
            end
        end
    end
    function addToRoot(obj)
        shName = obj.mJavaEditor.getShortName();
        shName = regexprep(char(shName),'\.m$','','once');

        daEditor_map = getappdata(0,'daEditor_map');
        map = containers.Map(shName,obj);
        if isa(daEditor_map,'containers.Map') && daEditor_map.isKey(shName)
            obj.releaseFromRoot();
        end
        map = [daEditor_map;map];
        setappdata(0,'daEditor_map',map);
    end

    function releaseFromRoot(obj,silentFlag)
        if nargin < 2 || isempty(silentFlag);   silentFlag = false; end
        shName = obj.mJavaEditor.getShortName();
        shName = regexprep(char(shName),'\.m$','','once');

        daEditor_map = getappdata(0,'daEditor_map');
        if isa(daEditor_map,'containers.Map') && daEditor_map.isKey(shName)
            oldObj = daEditor_map(shName);

            if oldObj.isInRoot()
                reset = true;
                oldObj.setKeyReleasedCallback(reset);
                oldObj.setMouseReleasedCallback(reset);
                oldObj.colorizeBreakPointBar(obj.COLOR_OFF);
            end

            delete(oldObj);
            if silentFlag
                warning('off','MATLAB:Containers:Map:NoKeyToRemove');
            end
            daEditor_map.remove(shName);
            if silentFlag
                warning('on','MATLAB:Containers:Map:NoKeyToRemove');
            end
            setappdata(0,'daEditor_map',daEditor_map);
            if ~silentFlag; fprintf('\nDaEditor "%s" released\n',shName);   end
        end
    end
end
methods (Static = true)
    function bool = isShortNameInRoot(shName)
        bool = false;

        shName = regexprep(char(shName),'\.m$','','once');
        daEditor_map = getappdata(0,'daEditor_map');
        if isa(daEditor_map,'containers.Map') && daEditor_map.isKey(shName)
            dE = daEditor_map(shName);
            bool = dE.isInRoot();
        end
    end
    function releaseAllDAEditors()
        daEditor_map = getappdata(0,'daEditor_map');
        if ~isempty(daEditor_map)
            silent = true;
            keys = daEditor_map.keys;
            for ii = 1:numel(keys)
                dE = daEditor_map(keys{ii});
                dE.releaseFromRoot(silent);
            end
        end
    end

    function newObj = getFromRoot(shName)
        daEditor_map = getappdata(0,'daEditor_map');
        if isa(daEditor_map,'containers.Map') && daEditor_map.isKey(shName)
            newObj = daEditor_map(shName);
        else
            newObj = NaN;
        end
    end
    
    function verifyObjects()
        daEditor_map = getappdata(0,'daEditor_map');
        if ~isempty(daEditor_map)
            silent = true;
            keys = daEditor_map.keys;
            for ii = 1:numel(keys)
                dE = daEditor_map(keys{ii});
                if isnumeric(dE.hEditorPane) && isinf(dE.hEditorPane)
                    dE.releaseFromRoot(silent);
                end
            end
        end
    end

    function longName = getActiveEditorLongName()
        edhandle = com.mathworks.mlservices.MLEditorServices;
        longName = char(edhandle.getEditorApplication.getActiveEditor.getLongName);
    end
    function shortName = getActiveEditorShortName()
        edhandle = com.mathworks.mlservices.MLEditorServices;
        shortName = char(edhandle.getEditorApplication.getActiveEditor.getShortName);
    end
end
end

