classdef DaEditorCallbacks < handle
    %DAEDITORCALLBACKS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        daEdit
    end
    
    properties (Access = private)
        idxCarets = [];
        idxOffset = 1;
    end
    
    properties (Hidden = true)
        latestKeyEvent
        latestMouseEvent 
    end
    
    methods
        function obj = DaEditorCallbacks(daEdit)
            if ~isa(daEdit,'daEditor.DaEditor'); error('hast to be an DaEditor object'); end
            obj.daEdit = daEdit; 
        end

        function multilineEdit(obj,evnt,rep)
            narginchk(2,3)
            if ~DA_getUserProfileSetting('MultiLineEditEnabled'); return; end
            if nargin < 3
                if any(evnt.getKeyCode == 16:17);    return; end
                rep = evnt.getKeyChar;
                if evnt.getKeyCode == evnt.VK_BACK_SPACE;   rep = '';   end
            end
            if ~isempty(rep)
                bsOffset = -1;
            else
                bsOffset = +1;
            end
            
            if isempty(obj.idxCarets);  return; end
            [obj.idxCarets,idx] = unique(obj.idxCarets,'rows');
            obj.idxCarets = flipud(obj.idxCarets);
            obj.idxOffset = obj.idxOffset(idx,:);
            zusColOffset = 0;
            for ii = 1:size(obj.idxCarets,1)
                if ii > 1;  idxB4 = idx; else idxB4 = [NaN,NaN]; end
                idx = obj.idxCarets(ii,:) + obj.idxOffset(ii,:);
                [r,c] = obj.daEdit.pos2idx(obj.daEdit.mJavaEditor.getCaretPosition);
                if c+bsOffset == idx(2) && r == idx(1)
                    continue;
                elseif idx(1) == idxB4(1)
                    zusColOffset = 1;
                end
                pos = obj.daEdit.idx2pos(idx(1),idx(2));
                if evnt.getKeyCode == evnt.VK_BACK_SPACE
                    obj.daEdit.repText(rep,pos-1,pos);
                else
                    obj.daEdit.repText(rep,pos,pos);
                end
            end
            if evnt.getKeyCode == evnt.VK_BACK_SPACE
                obj.idxOffset(ii,:) = obj.idxOffset(ii,:) - [0,1-zusColOffset];
            else
                obj.idxOffset(ii,:) = obj.idxOffset(ii,:) + [0,1+zusColOffset];
            end
        end
    end
     methods (Static) % callback
        function keyReleasedCallback(src,evnt,daEdit)
            % only add Shortcuts which are not already used by matlab itself.
            daEdit.callbacks.latestKeyEvent = evnt;
 
            ctrlFlag = evnt.isControlDown;
            shiftFlag = evnt.isShiftDown;
            altFlag = evnt.isAltDown;
            
            if ~isempty(daEdit.callbacks.idxCarets)
                if ~isempty(regexp(evnt.getKeyChar,'[^\n]','once')) || evnt.getKeyCode == evnt.VK_BACK_SPACE
                    daEdit.callbacks.multilineEdit(evnt);
                end
            else
                
            end
            daEditor.DaEditorCallbacks.runShortcutKey(src,evnt,daEdit);
        end
        
        function mouseReleasedCallback(src,evnt,daEdit)
            daEdit.callbacks.latestMouseEvent = evnt;
            daEdit.findAndReplace.resetSearchStr();
            
            if ~DA_getUserProfileSetting('MultiLineEditEnabled'); return; end
            
            if isempty(daEdit.callbacks.latestKeyEvent);    return; end
            
            if daEdit.callbacks.latestKeyEvent.isControlDown
                [r,c] = daEdit.pos2idx(daEdit.mJavaEditor.getCaretPosition);
                daEdit.callbacks.idxCarets = [daEdit.callbacks.idxCarets;r,c];
                daEdit.callbacks.idxOffset = [daEdit.callbacks.idxOffset;0,0];
            else
                daEdit.callbacks.idxCarets = [];
                daEdit.callbacks.idxOffset = [];
            end
%             daEdit.callbacks.idxCarets
        end
        
        function runShortcutKey(src, evnt, daEdit)
            % +----------------------------------------------------------------------------------------+
            % |  CTRL + SHIFT + ALT + E      -> schreibt das daEditor Objekt in den base workspace     |
            % |  CTRL + SHIFT + ALT + F      -> ffnet das erweiterte Suchen und Ersetzen Fenster      |
            % |  CTRL + SHIFT       + E      -> ffnet das Excel vom aktuellen SP - listdlg            |
            % |  CTRL + SHIFT       + L      -> ffnet das Loadmessdaten vom aktuellen SP - listdlg    |
            % |  CTRL + SHIFT       + T      -> updated das aktuelle SP - listdlg                      |
            % |  CTRL + SHIFT       + W      -> ffnet den Projekt pfad des Aktuellen SPs              |
            % |  CTRL + SHIFT       + Y      -> lscht die aktuelle zeile im editor - mehrere mglich  |
            % |  CTRL               + E      -> Ersatz von CTRL + D (mit extended Editor Funktionen    |
            % |                 ALT + INSERT -> (Einfg) ffnet das Script Block GUI                    |
            % |                 ALT + C      -> Chanellliste des SPs                                   |
            % |                 ALT + M      -> Messungsliste des SPs                                  |
            % |  CTRL +             + SPACE  -> Tabcompletion for MESRC Stuff. e.g. %Calc<CTRL+SPACE>  |
            % |  CTRL +             + F12    -> IntelliJ like  File Structure Search                   |
            % +----------------------------------------------------------------------------------------+
            
            % see also DA_UserProfileSettings.dispDocumentation
            ctrlFlag = evnt.isControlDown;
            shiftFlag = evnt.isShiftDown;
            altFlag = evnt.isAltDown;
            ctrlShiftFlag = ctrlFlag && shiftFlag && ~altFlag;
            ctrlShiftAltFlag = ctrlFlag && shiftFlag && altFlag;
            ctrlOnlyFlag = ctrlFlag && ~shiftFlag && ~altFlag;
            altOnlyFlag = ~ctrlFlag && ~shiftFlag && altFlag;
            
            %% CTRL + SHIFT + ALT + E
            if ctrlShiftAltFlag && evnt.getKeyCode == evnt.VK_E
                % schreibt in den base workspace eine variable des zugehrigen editor objekts
               assignin('base',...
                   ['daEditor_',regexp(char(daEdit.mJavaEditor.getShortName),'\w+(?=\.m)','match','once')],daEdit); 
            end
            %% CTRL + SHIFT + ALT + F
            if ctrlShiftAltFlag && evnt.getKeyCode == evnt.VK_F
                % ffnet das erweiterte Suchen und Ersetzen Fenster
                daEdit.findAndReplace.openGUI(daEdit);
            end   
            %% CTRL + SHIFT + L
            if  ctrlShiftFlag && evnt.getKeyCode == evnt.VK_L
                % ffnet ein LoadMessdaten mit dem im workspace vorhandenen SP
                [SP_WS,namesSP] = util.search4SP();
                if ~isempty(SP_WS)
                    cprintf([0,.5,1],'LoadMessdaten_ver2 %s\n',namesSP)
                    LoadMessdaten_ver2(SP_WS);
                else
                    cprintf([0,.5,1],'No SP found\n')
                end
            end
            %% CTRL + SHIFT + E
            if ctrlShiftFlag && evnt.getKeyCode == evnt.VK_E
                % ffnet das Excel von dem im workspace vorhandenen SP
                [SP_WS,namesSP] = util.search4SP();
                if ~isempty(SP_WS)
                    cprintf([0,.5,1],'Open Excel %s\n',namesSP)
                    wb = SP_getSPWorkbook(SP_WS);
                    wb.excelApp.setVisible;
                else
                    cprintf([0,.5,1],'No SP found\n')
                end
            end
            %% CTRL + SHIFT + T
            if ctrlShiftFlag && evnt.getKeyCode == evnt.VK_T
                % Excel to mat Update
                [SP_WS,namesSP] = util.search4SP();
                if ~isempty(SP_WS)
                    cprintf([0,.5,1],'Updating %s\n',namesSP)
                    SP_WS = SP_Excel2Mat_Update(SP_WS,1);
                    assignin('base', namesSP, SP_WS)
                else
                    cprintf([0,.5,1],'No SP found\n')
                end
            end
             %% CTRL + SHIFT + W
            if ctrlShiftFlag && evnt.getKeyCode == evnt.VK_W
                % open project path
                [SP_WS,namesSP] = util.search4SP();
                if ~isempty(SP_WS)
                    cprintf([0,.5,1],'Opening Project Path %s\n',namesSP)
                    winopen(SP_WS.LokalPath);
                else
                    cprintf([0,.5,1],'No SP found\n')
                end
            end
            %% CTRL + SHIFT + Y
            if ctrlShiftFlag && evnt.getKeyCode == evnt.VK_Y
                % remove Current Line
                linSize = numel(daEdit.getTextByLine(daEdit.selection(3)));
                if linSize == 0
                    offS = -2;
                    offE = -1;
                else
                    offS = -2;
                    offE = 0;
                end
                posS = daEdit.idx2pos(daEdit.selection(1),0)+offS;
                posE = daEdit.idx2pos(daEdit.selection(3),linSize)+offE;
                
                daEdit.repText('',posS,posE);
            end
            %% CTRL + E
            if ctrlOnlyFlag && evnt.getKeyCode == evnt.VK_E
                % berladenes CTRL + D von Matlab
                str = daEdit.mEditor.SelectedText;
                fprintf('Strg+D: "%s" in "%s"\n',str, char(daEdit.mJavaEditor.getShortName))
                edit(str)
                % mEditor.goToFunction('doLookAndFeel')
            end
            %% ALT + INSERT
            if altOnlyFlag && evnt.getKeyCode == evnt.VK_INSERT
                % Opens Script Block GUI
                MESR_GUI;
            end
            %% ALT + C
            if altOnlyFlag && evnt.getKeyCode == evnt.VK_C
%                 linTxt = daEdit.getTextByLine(daEdit.selection(3));
%                 possibleSP = regexp(linTxt,'\<(Bm|Ft|St|SP|La)[^_][\w]+\>','match','once'); % noch keine ahnung wie das mit mehreren funktionieren soll
                sp = util.search4SP();
                if isempty(sp); fprintf('\nOperation not possible "ALT+C"\n'); end
                daEditor.listdlgMC(sp,'c');
            end
            %% ALT + M
            if altOnlyFlag && evnt.getKeyCode == evnt.VK_M
%                 linTxt = daEdit.getTextByLine(daEdit.selection(3));
%                 possibleSP = regexp(linTxt,'\<(Bm|Ft|St|SP|La)[^_][\w]+\>','match','once'); % noch keine ahnung wie das mit mehreren funktionieren soll
                sp = util.search4SP();
                if isempty(sp); fprintf('\nOperation not possible "ALT+M"\n'); end
                daEditor.listdlgMC(sp,'m');
            end
            %% CTRL + SPACE
            if ctrlOnlyFlag && evnt.getKeyCode == evnt.VK_SPACE
                aE = matlab.desktop.editor.getActive;
                daEditor.DaEditorCallbacks.tabCompletion(aE);
            end
            %% CTRL + F12
            if ctrlOnlyFlag && evnt.getKeyCode == evnt.VK_F12
                fs = daEditor.FileStructure.getInstance;
                fs.setEditorObj(daEdit);
            end
        end

        function tabCompletion(aE)
            pos = aE.Selection;
            pos(2,1:4) = pos;
            cursorPos = matlab.desktop.editor.positionInLineToIndex(aE, pos(1,1),pos(1,2));
            txt = aE.Text(matlab.desktop.editor.positionInLineToIndex(aE,pos(1,1),1):cursorPos-1);

            idx = regexp(txt,'\%\S'); % no space allowed after "%"
            if isempty(idx); return; end
            idx = idx(end); % nearest to cursor
            expr = txt(idx+1:end);

            mesrPath = fullfile(SDS_ToolPath_DA,'MatlabM\Tools\Matlab_Editor_StringReplacements\Replacements');
            oCD = cd(mesrPath);
            dirCont = dir('MESRC_*.m');
            mesrList = {dirCont.name};
            mesrList = mesrList(:);
            mesrList = regexprep(mesrList,'MESRC_(.*?)\.m','$1');
            cd(oCD);

            mesrListTab = mesrList(util.regexCell(mesrList,expr));
            txts = strjoin(mesrListTab,'\n\t');
            fprintf('\n-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=');
            fprintf('\nTabCompletion on MESRC:')
            fprintf('\n\t%s',txts)
            fprintf('\n-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=-=~=');
            
%             xy = Aid_getMousePosition()
%             jWindow = javax.swing.JWindow;
% %             jWindow.getContentPane().add(component, BorderLayout.CENTER);
%             jWindow.setLocation(xy(1), xy(2))
%             jWindow.setSize(300, 300);
%             jWindow.setVisible(true);
%             jWindow.setVisible(false);
        end
     end
end

