classdef (Sealed) FileStructure < handle
    %FILESTRUCTURE Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (Access = private)
        daEdit % daEditor object
        f
        eSearch
        jEd
        cbInherited
        lb

        dataCell@cell % 1 col: displayed string in listbox
                      % 2 col: function name
                      % 3 col: line
                      % 4 col: file shortname
                      % 5 col: file longname
        dataCellFilt
        lastClicked = java.lang.System.currentTimeMillis();

        DISP_STRING = 1;
        DISP_FUNCTION_NAME = 2;
        DISP_LINE = 3;
        DISP_FILESHORT = 4;
        DISP_FILELONG = 5;
    end
    
    methods (Access = private)
        function obj = FileStructure()
            obj.f = figure('menubar','none','units','pixels','name','File Structure of $MFILE$','numbertitle','off',...
                'resize','off',...
                'KeyPressFcn',@(f,e) daEditor.FileStructure.keyPressFcn(obj,f,e) );
            % edit Searchbox
            fPos = obj.f.Position;
            obj.eSearch = uicontrol('style','edit', 'position',[10,fPos(4)-30,300,20],...
                'HorizontalAlignment','left','KeyPressFcn',@(f,e) daEditor.FileStructure.keyPressFcn(obj,f,e) );
            obj.jEd = findjobj(obj.eSearch);
            % checkbox include inherited members
            ePos = obj.eSearch.Position;
            obj.cbInherited = uicontrol('style','checkbox', 'position',[ePos(1)+ePos(3)+10,ePos(2),220,ePos(4)],...
                'String','show inherited members (CTRL + F12)');
            obj.cbInherited.Enable = 'off'; % TODO:
            % listbox
            obj.lb = uicontrol('style','listbox','position',[ePos(1),10,fPos(3)-ePos(1)-8,ePos(2)-20],...
                'Callback',@(f,e) daEditor.FileStructure.buttonDownFcn(obj,f,e));
        end
        function bool = isDoubleClicked(obj)
            bool = false;
            if (java.lang.System.currentTimeMillis() - obj.lastClicked) < 300
                bool = true;
            end
            obj.lastClicked = java.lang.System.currentTimeMillis();
        end
    end
    methods (Access = public)
        function obj = setEditorObj(obj,daEdit)
            shName = daEdit.getShortName();
            obj.f.Name = regexprep(obj.f.Name,'\$MFILE\$',shName);
            obj.daEdit = daEdit;
            
            methsPos = daEdit.getTextPosByExpr('methods');
            if ~isempty(methsPos)
                lineMethods = daEdit.pos2idx(methsPos);
                meths = daEdit.getTextByLine(lineMethods);
                if ~iscell(meths); meths = {meths}; end
                invalidMeths = ~util.regexCell(meths,'^\s*methods($|\s|\()');
                meths(invalidMeths) = []; lineMethods(invalidMeths) = [];
                meths = regexprep(meths,'^\s*methods\s*','');
            end

            functsPos = daEdit.getTextPosByExpr('function');
            if ~isempty(functsPos)
                lineFunctions = daEdit.pos2idx(functsPos);
                functs = daEdit.getTextByLine(lineFunctions);
                if ~iscell(functs); functs = {functs}; end
                invalidFuncts = ~util.regexCell(functs,'^\s*function($|\s|\()');
                functs(invalidFuncts) = []; lineFunctions(invalidFuncts) = [];
                functs = regexprep(functs,'^\s*function\s*','');
            else
                obj.lb.String = 'no functions';
                obj.dataCell(:,obj.DISP_STRING) = 'no functions';
                obj.dataCell(:,obj.DISP_FUNCTION_NAME) = '';
                obj.dataCell(:,obj.DISP_LINE) = '';
                obj.dataCell(:,obj.DISP_FILESHORT) = '';
                obj.dataCell(:,obj.DISP_FILELONG) = '';
                obj.dataCellFilt = obj.dataCell;
                return;
            end
            
            txtC = cell(size(functs));
            for ii = 1:numel(txtC)
                if ~isempty(methsPos) && ~isempty(lineMethods)
                    mIdx = find(lineMethods < lineFunctions(ii),1,'last');
                    meth = meths{mIdx};
                    mAccess = regexp(meth,'(?<=Access\s*=\s*)\w+','match','once');
                    switch lower(mAccess)
                        case 'private'; col = grafik.Color(0.4,0,0).toHex();
                        case 'public'; col = grafik.Color(0,0.4,0).toHex();
                        case 'protected'; col = grafik.Color(1,0.6,0).toHex();
                        otherwise; col = grafik.Color(0,0,0).toHex();
                    end
                    mAccess_ = sprintf('<FONT color="#%s">%s</FONT>',col,mAccess);
                    meth = regexprep(meth,mAccess,mAccess_);
                    txtC{ii} = sprintf('<html>(%5g) %s -> %s</html>',lineFunctions(ii),functs{ii},meth);
                else
                    txtC{ii} = sprintf('(%5g) %s',lineFunctions(ii),functs{ii});
                end
            end
            obj.lb.String = txtC;
            obj.dataCell = {};
            obj.dataCell(:,obj.DISP_STRING) = txtC;
            obj.dataCell(:,obj.DISP_FUNCTION_NAME) = functs;
            obj.dataCell(:,obj.DISP_LINE) = num2cell(lineFunctions);
            obj.dataCell(:,obj.DISP_FILESHORT) = repmat({shName(1:end-2)},size(functs));
            obj.dataCell(:,obj.DISP_FILELONG) = repmat({daEdit.getActiveEditorLongName()},size(functs));
            obj.dataCellFilt = obj.dataCell;
        end
    end
    methods (Static = true)
        function instance = getInstance()
         persistent localObj
         if isempty(localObj) || ~isvalid(localObj) || ~isvalid(localObj.f)
            localObj = daEditor.FileStructure;
         end
         instance = localObj;

         instance.f.Visible = 'on';
         figure(instance.f);         
         instance.jEd.requestFocus();
      end
    end

    methods (Static = true, Access = private)
        function keyPressFcn(obj, f,e)
            % Escape
            if strcmpi(e.Key,'escape')
                obj.f.Visible = 'off';
            end
            % CTRL + F12
            if ~isempty(e.Modifier) && all(util.regexCell(e.Modifier,'control')) && strcmpi(e.Key,'f12')
                obj.cbInherited.Value = ~obj.cbInherited.Value;
            end
            if ~isempty(e.Character)
                idx = util.regexCell(obj.dataCell(:,1),char(obj.jEd.getText));
                if ~any(idx)
                    if numel(char(obj.jEd.getText)) < 1
                        obj.dataCellFilt = obj.dataCell;
                    else
                        obj.dataCellFilt = {};
                        obj.dataCellFilt(obj.DISP_STRING) = {sprintf('nothing found with expr "%s"',char(obj.jEd.getText))};
                    end
                else
                    obj.dataCellFilt = obj.dataCell(idx,:);
                end
                if obj.lb.Value > numel(obj.dataCellFilt(:,1))
                    % takes care of:
                    %   Warning: Single-selection 'listbox' control requires that 'Value' be an integer within String range
                    %   Control will not be rendered until all of its parameter values are valid
                    obj.lb.Value = 1;
                end
                obj.lb.String = obj.dataCellFilt(:,obj.DISP_STRING);
            end

            if all(util.regexCell(char(e.Key),'downarrow'))
                if obj.lb.Value < numel(obj.dataCellFilt(:,1))
                    obj.lb.Value = obj.lb.Value + 1;
                end
            end
            if all(util.regexCell(char(e.Key),'uparrow'))
                if obj.lb.Value > 1
                    obj.lb.Value = obj.lb.Value - 1;
                end
            end
            if all(util.regexCell(char(e.Key),'return'))
                daEditor.FileStructure.buttonDownFcn(obj,obj.lb,[],true);
            end
        end
        function buttonDownFcn(obj, l,e, forceClicked)
            if nargin < 4 || isempty(forceClicked); forceClicked = false; end
            if forceClicked || obj.isDoubleClicked()
                if numel(obj.dataCellFilt) == 1; return; end
                l = obj.dataCellFilt{l.Value,obj.DISP_LINE};
                obj.daEdit.goToLine(l);
                obj.f.Visible = 'off';
            end
        end
    end
    
end

